//---------------------------------------------------------------------------

#pragma hdrstop
#include "Date.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)

namespace ipt{
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

Date::Date()
{
  time_t time0 = ::time(0);
  struct ::tm *now = ::localtime(&time0);
  count = Count(now->tm_year + 1900, now->tm_mon + 1, now->tm_mday);
}

Date::Date(const Date& other):
  count(other.count)
{
}

Date::Date(int year, int month, int day):
  count(Count(year, month, day))
{
}

Date::~Date()
{
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

int Date::Year() const
{
  int c = Century();
  int days = count - FirstDayOfCentury(c);
  return FirstYearOfCentury(c) +
    days / fourYears * 4 + min(days % fourYears / 365, 3);
}

int Date::Month() const
{
  bool leap = InLeapYear();
  int dayNumber = DayNumber();
  int result = 1;
  while (accumulated[leap][result-1] < dayNumber)
    result++;
  return result;
}

int Date::Day() const
{
  return count - FirstDayOfMonth(Year(), Month()) + 1;
}

int Date::Century() const
{
//return firstCentury + Centuries(count);
  return firstCentury + 
    count / fourCenturies * 4 + min(count % fourCenturies / oneCentury, 3);
}

int Date::DayNumber() const
{
  return count - FirstDayOfYear(Year()) + 1;
}
  
int Date::Count() const
{
  return count;
}

int Date::Month(int week, int year)
{
  Date d(year,1,1);
  d.ReachForward(Date::SUNDAY);
  d.AddWeek(week-1);
  return d.Month();

}

int Date::WeekNumber() const
{
  Date d(Year(),1,1);
  d.ReachBackward(Date::MONDAY);
  int result=1;
  for(int i=0;i+7<=DaysSince(d);i+=7)
        result++;
  return result;
}

int Date::Moon() const
{
Date d(2004,6,3);
for (int i=1;i<5;i++){
   if (i==2)
      d.Set(2004,6,9);
   if (i==3)
      d.Set(2004,6,17);
   if (i==4)
      d.Set(2004,6,25);

   if (d==*this)
      return i;
   else{
   if (d<*this){
      while (d<*this)
         d.AddWeek(4);
         if(d==*this)
            return i;
   }
   else {
      while (d>*this)
         d.SubtractWeek(4);
         if(d==*this)
            return i;
   }
   }
}
return 0;

}  

void Date::Set(int newYear, int newMonth, int newDay)
{
  count = Count(newYear, newMonth, newDay);
}

void Date::Set(int year, int dayNumber)
{
  count = Count(year, 1, 1) + dayNumber - 1;
}
//---------------------------------------------------------------------------
void Date::Forth()
{
  count++;
}

void Date::Back()
{
  count--;
}

void Date::Add(int x)
{
  count += x;
}

void Date::Subtract(int x)
{
  count -= x;
}

int Date::DaysTo(const Date& other) const
{
  return other.count - this->count;
}

int Date::DaysSince(const Date& other) const
{
  return this->count - other.count;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

Date::WeekdayType Date::Weekday() const
{
  return static_cast<WeekdayType>((count + firstWeekday) % 7);
}
void Date::AddWeek(int x)
{
  Add(7 * x);
}

void Date::SubtractWeek(int x)
{
  Subtract(7 * x);
}

void Date::ReachForward(WeekdayType weekday)
{
  Add((7 + weekday - Weekday()) % 7);
}

void Date::ReachBackward(WeekdayType weekday)
{
  Subtract((7 + Weekday() - weekday) % 7);
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void Date::Write(ostream& output) const
{
  output << Year() << " " << Month() << " " << Day();
}

void Date::WriteLine(ostream& output) const
{
  Write();
  output << endl;
}

void Date::Read(istream& input)
{
  int y;
  int m;
  int d;
  input >> y >> m>> d;
  Set(y, m, d);
}

istream& operator >> (istream& input,  Date& d)
{
  d.Read(input);
  return input;
}

ostream& operator << (ostream& output, const Date& d)
{
  d.Write(output);
  return output;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int Date::MonthSize() const
{
  return DaysInMonth(Year(), Month());
}

int Date::YearSize() const
{
  return DaysInYear(Year());
}

int Date::CenturySize() const
{
  return DaysInCentury(Century());
}

bool Date::InLeapYear() const
{
  return LeapYear(Year());
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
 bool Date::operator == (const Date& other) const
{
  return this->count == other.count;
}

bool Date::operator != (const Date& other) const
{
  return this->count != other.count;
}

bool Date::operator <= (const Date& other) const
{
  return this->count <= other.count;
}

bool Date::operator <  (const Date& other) const
{
  return this->count < other.count;
}

bool Date::operator >= (const Date& other) const
{
  return this->count >= other.count;
}

bool Date::operator >  (const Date& other) const
{
  return this->count > other.count;
}

Date& Date::operator ++  (int)
{
  count++;
  return *this;
}

Date& Date::operator --  (int)
{
  count--;
  return *this;
}

Date Date::operator ++  ()
{
  Date temp(*this);
  count++;
  return temp;

}

Date Date::operator --  ()
{
  Date temp(*this);
  count--;
  return temp;
}


Date& Date::operator += (int x)
{
  count += x;
  return *this;
}

Date& Date::operator -= (int x)
{
  count -= x;
  return *this;
}

Date Date::operator + (int x) const
{
  Date result(*this);
  result += x;
  return result;
}

Date Date::operator - (int x) const
{
  Date result(*this);
  result -= x;
  return result;
}

int Date::operator - (const Date& other) const
{
  return this->count - other.count;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int Date::Count(int y, int m, int d)
{
  return FirstDayOfMonth(y, m) + d - 1;
}

bool Date::LeapYear(int y)
{
  return y % 400 == 0 || y % 4 == 0  && y % 100 != 0;
}

int Date::DaysInMonth(int y, int m)
{
  return daysInMonth[LeapYear(y)][m - 1];
} 

int Date::DaysInYear(int y)
{
  return 365 + LeapYear(y);
}

int Date::CenturyOf(int y)
{
  return (y - 1) / 100 + 1;
}

int Date::DaysInCentury(int c)
{
  return 365 * 100 + 25 - (c % 4 != 0);
}

int Date::FirstYearOfCentury(int c)
{
  return (c - 1) * 100 + 1;
}

int Date::FirstDayOfCentury(int c)
{
  return (c - firstCentury) / 4 * fourCenturies + (c - firstCentury) % 4 * oneCentury;
}

int Date::FirstDayOfYear(int y)
{
  return FirstDayOfCentury(CenturyOf(y)) +
             (y - 1) % 100 / 4 * fourYears + (y - 1) % 4 * 365;
}

int Date::FirstDayOfMonth(int y, int m)
{
  bool leap = LeapYear(y);
  return FirstDayOfYear(y) + (accumulated[leap][m - 1] - daysInMonth[leap][m - 1]);
}

int Date::YearsInCentury(int d)
{
  return d / fourYears * 4 + min(d % fourYears / 365, 3);
}

//---------------------------------------------------------------------------
//------ initialization of static variables ------
//---------------------------------------------------------------------------
const int Date::firstYear = 1601;
const int Date::firstCentury = 17;
const Date Date::first(Date::firstYear, 1, 1);
const Date::WeekdayType Date::firstWeekday = MONDAY;
const int Date::daysInMonth[2][12] =
                {{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
                  {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}};
const int Date::accumulated[2][12] =
                {{31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365},
                  {31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366}};
const int Date::fourYears = 365*4 + 1;                                  //    1461
const int Date::oneCentury = fourYears * 25 - 1;                  //  36524
const int Date::fourCenturies = oneCentury * 4 + 1;            // 146097
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
bool Date::GoodYear()const
{
   Date ini(Year() , 1 , 1);
   Date fin(Year() , 12 , 31);
   int numFridays=0;
   while(ini != fin)
   {
       if(ini.Weekday() == FRIDAY && ini.Day() == 13)
           numFridays++;
       ini++;
   }
   return numFridays >= 3;
}
bool Date::Weekend()const
{
   return Weekday() == SATURDAY ||
          Weekday() == SUNDAY; 

}
//---------------------------------------------------------------------------
}//namespace
